#ifndef __PVS_H
#define __PVS_H

	
namespace CHCDemoEngine
{

/** Template class representing the Potentially Visible Set (PVS) 
	mainly from a view cell, but also e.g., from objects.
*/
class Pvs
{

public:

	Pvs(): mEntries() {

		entries.set_deleted_key(-1);
		entries.set_empty_key(-2);

	};

	int GetSize() const {
		return (int)entries.size();
	}


	bool Empty() const {
		return entries.empty();
	}

	/** Adds object to PVS.
	@returns contribution of sample (0 or 1)
	*/
	bool Insert(const int id, const float time = 0.0f) {

		pair<PVS_ITERATOR, bool> result = entries.insert(pair<int, float>(id,time));

		return result.second;
	}

	bool Find(const int id) {
		PVS_ITERATOR it;
		return Find(id, it);
	}

	bool Find(const int id, PVS_ITERATOR &it) {
		it = entries.find(id);

		// already in map
		return (it != entries.end());
	}

	void Clear() {
		entries.clear();
	}

	static void Merge(Pvs &mergedPvs,
		const Pvs &a, 
		const Pvs &b)
	{
		mergedPvs.entries.insert(a.entries.begin(), a.entries.end());
		mergedPvs.entries.insert(b.entries.begin(), b.entries.end());
	}

	// just an estimate - could not find proper reference how to
	// calculate memory occupied by hash_map and hash_set!
	int GetMemoryUsage() const {

		return sizeof(Pvs) + sizeof(HASH_SET::value_type)*entries.bucket_count();
	}

protected:
	/// hash table of PVS entries
	HASH_SET entries; 

};


/** Iterator over a hash pvs.
*/
class PvsIterator
{
protected:

	CONST_PVS_ITERATOR itCurrent;
	CONST_PVS_ITERATOR itEnd;

public:

	PvsIterator() {}

	PvsIterator(Pvs &pvs):
	itCurrent(pvs.entries.begin()), itEnd(pvs.entries.end())
	{
	}

	bool HasMoreEntries() const 
	{
		return (itCurrent != itEnd);
	}

	void Next() {
		++itCurrent;
	}

	int Current() const {

		return (*itCurrent).first;

	}
	float CurrentTime() const {

		return (*itCurrent).second;

	}

};

#if 0
// iterator of the PVS at the given time stamp
// onnly works properly in PVS_TIME_STAMPS is defined
// otherwise behaves the same as PvsIterator

class PvsTimeIterator : public PvsIterator
{
protected:
	float time;
public:

	PvsTimeIterator(Pvs &pvs, const float _time):
	  PvsIterator(pvs), time(_time)
	  {
		  // find the first entry satisfying the time constraint
		  for (;HasMoreEntries(); ++itCurrent)
			  if ((*itCurrent).second <= time)
				  break;
	  }

	  void Next() {
		  for (++itCurrent; HasMoreEntries(); ++itCurrent)
			  if ((*itCurrent).second <= time)
				  break;
	  }

};

#endif
}

#endif

